/* Copyright (c) 2013-2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include "xbt.h"

XBT_LOG_NEW_DEFAULT_SUBCATEGORY(smpi_colls, smpi,
                                "Logging specific to SMPI collectives");
