/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.11
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.simgrid.surf;

public final class ActionState {
  public final static ActionState SURF_ACTION_READY = new ActionState("SURF_ACTION_READY", SurfJNI.SURF_ACTION_READY_get());
  public final static ActionState SURF_ACTION_RUNNING = new ActionState("SURF_ACTION_RUNNING");
  public final static ActionState SURF_ACTION_FAILED = new ActionState("SURF_ACTION_FAILED");
  public final static ActionState SURF_ACTION_DONE = new ActionState("SURF_ACTION_DONE");
  public final static ActionState SURF_ACTION_TO_FREE = new ActionState("SURF_ACTION_TO_FREE");
  public final static ActionState SURF_ACTION_NOT_IN_THE_SYSTEM = new ActionState("SURF_ACTION_NOT_IN_THE_SYSTEM");

  public final int swigValue() {
    return swigValue;
  }

  public String toString() {
    return swigName;
  }

  public static ActionState swigToEnum(int swigValue) {
    if (swigValue < swigValues.length && swigValue >= 0 && swigValues[swigValue].swigValue == swigValue)
      return swigValues[swigValue];
    for (int i = 0; i < swigValues.length; i++)
      if (swigValues[i].swigValue == swigValue)
        return swigValues[i];
    throw new IllegalArgumentException("No enum " + ActionState.class + " with value " + swigValue);
  }

  private ActionState(String swigName) {
    this.swigName = swigName;
    this.swigValue = swigNext++;
  }

  private ActionState(String swigName, int swigValue) {
    this.swigName = swigName;
    this.swigValue = swigValue;
    swigNext = swigValue+1;
  }

  private ActionState(String swigName, ActionState swigEnum) {
    this.swigName = swigName;
    this.swigValue = swigEnum.swigValue;
    swigNext = this.swigValue+1;
  }

  private static ActionState[] swigValues = { SURF_ACTION_READY, SURF_ACTION_RUNNING, SURF_ACTION_FAILED, SURF_ACTION_DONE, SURF_ACTION_TO_FREE, SURF_ACTION_NOT_IN_THE_SYSTEM };
  private static int swigNext = 0;
  private final int swigValue;
  private final String swigName;
}

