/* Copyright (c) 2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef PLUGINS_H_
#define PLUGINS_H_

SG_BEGIN_DECL()

/** \ingroup SURF_plugins
 *  \brief The cpu energy consumption plugin
 */
XBT_PUBLIC(void) sg_energy_plugin_init(void);

SG_END_DECL()

#endif /* PLUGINS_H_ */
