/* simgrid.h - Public interface all SimGrid APIs                            */

/* Copyright (c) 2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef SIMGRID_H
#define SIMGRID_H

#include <xbt.h>

#include <simgrid/datatypes.h>
#include <simgrid/modelchecker.h>
#include <simgrid/platf_generator.h>
#include <simgrid/platf.h>
#include <simgrid/simix.h>
#include <simgrid/plugins.h>

// SG_BEGIN_DECL()
// nothing
// SG_END_DECL()

#endif                          /* SG_PLATF_H */
