/* Copyright (c) 2011-2012, 2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#include <stdio.h>
#include "xbt/log.h"
XBT_LOG_NEW_DEFAULT_CATEGORY(HelloWorld,
                             "Messages specific for this Hello World example");
void main()
{
  XBT_INFO("Hello World !!!");
}
