/*********************************************
 * File Generated by src/simix/simcalls.py   *
 *                from src/simix/simcalls.in *
 * Do not modify this file, add new simcalls *
 * in src/simix/simcalls.in                  *
 *********************************************/

static inline smx_host_t simcall_host_get_by_name__get__result(smx_simcall_t simcall){
  return (smx_host_t) simcall->result.dp;
}
static inline void simcall_host_get_by_name__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline const char* simcall_host_get_name__get__result(smx_simcall_t simcall){
  return  simcall->result.cc;
}
static inline void simcall_host_get_name__set__result(smx_simcall_t simcall, const char* result){
    simcall->result.cc = result;
}




static inline xbt_dict_t simcall_host_get_properties__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_host_get_properties__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline int simcall_host_get_core__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_host_get_core__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline xbt_swag_t simcall_host_get_process_list__get__result(smx_simcall_t simcall){
  return (xbt_swag_t) simcall->result.dp;
}
static inline void simcall_host_get_process_list__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline double simcall_host_get_speed__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_get_speed__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline double simcall_host_get_available_speed__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_get_available_speed__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline int simcall_host_get_state__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_host_get_state__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline double simcall_host_get_current_power_peak__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_get_current_power_peak__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline double simcall_host_get_power_peak_at__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_get_power_peak_at__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline int simcall_host_get_nb_pstates__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_host_get_nb_pstates__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}


static inline double simcall_host_get_consumed_energy__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_get_consumed_energy__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline smx_action_t simcall_host_execute__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_host_execute__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_action_t simcall_host_parallel_execute__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_host_parallel_execute__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}




static inline double simcall_host_execution_get_remains__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_host_execution_get_remains__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline int simcall_host_execution_get_state__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_host_execution_get_state__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}






static inline int simcall_host_execution_wait__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_host_execution_wait__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline xbt_dict_t simcall_host_get_mounted_storage_list__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_host_get_mounted_storage_list__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline xbt_dynar_t simcall_host_get_attached_storage_list__get__result(smx_simcall_t simcall){
  return (xbt_dynar_t) simcall->result.dp;
}
static inline void simcall_host_get_attached_storage_list__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}




static inline void* simcall_vm_create__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_vm_create__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline int simcall_vm_get_state__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_vm_get_state__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}


static inline void* simcall_vm_get_pm__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_vm_get_pm__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}






























static inline int simcall_process_count__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_count__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_process_get_PID__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_get_PID__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_process_get_PPID__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_get_PPID__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline void* simcall_process_get_data__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_process_get_data__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline smx_host_t simcall_process_get_host__get__result(smx_simcall_t simcall){
  return (smx_host_t) simcall->result.dp;
}
static inline void simcall_process_get_host__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline const char* simcall_process_get_name__get__result(smx_simcall_t simcall){
  return  simcall->result.cc;
}
static inline void simcall_process_get_name__set__result(smx_simcall_t simcall, const char* result){
    simcall->result.cc = result;
}
static inline int simcall_process_is_suspended__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_is_suspended__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline xbt_dict_t simcall_process_get_properties__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_process_get_properties__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline int simcall_process_join__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_join__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_process_sleep__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_process_sleep__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}




static inline smx_process_t simcall_process_restart__get__result(smx_simcall_t simcall){
  return (smx_process_t) simcall->result.dp;
}
static inline void simcall_process_restart__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_rdv_t simcall_rdv_create__get__result(smx_simcall_t simcall){
  return (smx_rdv_t) simcall->result.dp;
}
static inline void simcall_rdv_create__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline unsigned int simcall_rdv_comm_count_by_host__get__result(smx_simcall_t simcall){
  return  simcall->result.ui;
}
static inline void simcall_rdv_comm_count_by_host__set__result(smx_simcall_t simcall, unsigned int result){
    simcall->result.ui = result;
}
static inline smx_action_t simcall_rdv_get_head__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_rdv_get_head__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline smx_process_t simcall_rdv_get_receiver__get__result(smx_simcall_t simcall){
  return (smx_process_t) simcall->result.dp;
}
static inline void simcall_rdv_get_receiver__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_action_t simcall_comm_iprobe__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_comm_iprobe__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline smx_action_t simcall_comm_isend__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_comm_isend__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline smx_action_t simcall_comm_irecv__get__result(smx_simcall_t simcall){
  return (smx_action_t) simcall->result.dp;
}
static inline void simcall_comm_irecv__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}


static inline int simcall_comm_waitany__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_comm_waitany__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}


static inline int simcall_comm_test__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_comm_test__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_comm_testany__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_comm_testany__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline double simcall_comm_get_remains__get__result(smx_simcall_t simcall){
  return  simcall->result.d;
}
static inline void simcall_comm_get_remains__set__result(smx_simcall_t simcall, double result){
    simcall->result.d = result;
}
static inline int simcall_comm_get_state__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_comm_get_state__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline void* simcall_comm_get_src_data__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_comm_get_src_data__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline void* simcall_comm_get_dst_data__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_comm_get_dst_data__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_process_t simcall_comm_get_src_proc__get__result(smx_simcall_t simcall){
  return (smx_process_t) simcall->result.dp;
}
static inline void simcall_comm_get_src_proc__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_process_t simcall_comm_get_dst_proc__get__result(smx_simcall_t simcall){
  return (smx_process_t) simcall->result.dp;
}
static inline void simcall_comm_get_dst_proc__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline smx_mutex_t simcall_mutex_init__get__result(smx_simcall_t simcall){
  return (smx_mutex_t) simcall->result.dp;
}
static inline void simcall_mutex_init__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}




static inline int simcall_mutex_trylock__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_mutex_trylock__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}


static inline smx_cond_t simcall_cond_init__get__result(smx_simcall_t simcall){
  return (smx_cond_t) simcall->result.dp;
}
static inline void simcall_cond_init__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}










static inline smx_sem_t simcall_sem_init__get__result(smx_simcall_t simcall){
  return (smx_sem_t) simcall->result.dp;
}
static inline void simcall_sem_init__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}




static inline int simcall_sem_would_block__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_sem_would_block__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}




static inline int simcall_sem_get_capacity__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_sem_get_capacity__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline sg_size_t simcall_file_read__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_file_read__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline sg_size_t simcall_file_write__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_file_write__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline smx_file_t simcall_file_open__get__result(smx_simcall_t simcall){
  return (smx_file_t) simcall->result.dp;
}
static inline void simcall_file_open__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline int simcall_file_close__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_file_close__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_file_unlink__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_file_unlink__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline sg_size_t simcall_file_get_size__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_file_get_size__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline sg_size_t simcall_file_tell__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_file_tell__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline int simcall_file_seek__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_file_seek__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline xbt_dynar_t simcall_file_get_info__get__result(smx_simcall_t simcall){
  return (xbt_dynar_t) simcall->result.dp;
}
static inline void simcall_file_get_info__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline int simcall_file_move__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_file_move__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline sg_size_t simcall_storage_get_free_size__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_storage_get_free_size__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline sg_size_t simcall_storage_get_used_size__get__result(smx_simcall_t simcall){
  return  simcall->result.sgsz;
}
static inline void simcall_storage_get_used_size__set__result(smx_simcall_t simcall, sg_size_t result){
    simcall->result.sgsz = result;
}
static inline xbt_dict_t simcall_storage_get_properties__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_storage_get_properties__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline xbt_dict_t simcall_storage_get_content__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_storage_get_content__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline xbt_dict_t simcall_asr_get_properties__get__result(smx_simcall_t simcall){
  return (xbt_dict_t) simcall->result.dp;
}
static inline void simcall_asr_get_properties__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
#ifdef HAVE_LATENCY_BOUND_TRACKING
static inline int simcall_comm_is_latency_bounded__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_comm_is_latency_bounded__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
#endif

#ifdef HAVE_TRACING


#endif

#ifdef HAVE_MC
static inline void* simcall_mc_snapshot__get__result(smx_simcall_t simcall){
  return  simcall->result.dp;
}
static inline void simcall_mc_snapshot__set__result(smx_simcall_t simcall, void* result){
    simcall->result.dp = result;
}
static inline int simcall_mc_compare_snapshots__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_mc_compare_snapshots__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
static inline int simcall_mc_random__get__result(smx_simcall_t simcall){
  return  simcall->result.i;
}
static inline void simcall_mc_random__set__result(smx_simcall_t simcall, int result){
    simcall->result.i = result;
}
#endif
