/* Copyright (c) 2013-2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef SMPI_COLLS_PRIVATE_H
#define SMPI_COLLS_PRIVATE_H

#include "colls.h"

XBT_LOG_EXTERNAL_DEFAULT_CATEGORY(smpi_colls);

#endif
