/* ----------------------------------------------------------------------------
 * This file was automatically generated by SWIG (http://www.swig.org).
 * Version 2.0.11
 *
 * Do not make changes to this file unless you know what you are doing--modify
 * the SWIG interface file instead.
 * ----------------------------------------------------------------------------- */

package org.simgrid.surf;

public class Surf {
  public static Object getAction(long jarg1) {
    return SurfJNI.getAction(jarg1);
  }

  public static Object getCpuModelDirector(long jarg1) {
    return SurfJNI.getCpuModelDirector(jarg1);
  }

  public static Object getCpuDirector(long jarg1) {
    return SurfJNI.getCpuDirector(jarg1);
  }

  public static Object getCpuActionDirector(long jarg1) {
    return SurfJNI.getCpuActionDirector(jarg1);
  }

  
  /**
    * @return The current simulated time
    */
  public static double getClock() {
    return SurfJNI.getClock();
  }

  public static void clean() {
    SurfJNI.clean();
  }

  public static CpuModel getCpuModel() {
  long cPtr = SurfJNI.getCpuModel();
  return (CpuModel)Surf.getCpuModelDirector(cPtr);
}

  public static void setCpuModel(CpuModel cpuModel) {
    SurfJNI.setCpuModel(CpuModel.getCPtr(cpuModel), cpuModel);
  }

  public static void setCpu(String name, Cpu cpu) {
    SurfJNI.setCpu(name, Cpu.getCPtr(cpu), cpu);
  }

  public static NetworkLink[] getRoute(String srcName, String dstName) {
     return NetworkLink.cArrayWrap(SurfJNI.getRoute(srcName, dstName), false);
}

}
