/* Copyright (c) 2012, 2014. The SimGrid Team.
 * All rights reserved.                                                     */

/* This program is free software; you can redistribute it and/or modify it
 * under the terms of the license (GNU LGPL) which comes with this package. */

#ifndef _BUGGED1_LIVENESS_H
#define _BUGGED1_LIVENESS_H

int predR(void);
int predCS(void);

int coordinator(int argc, char *argv[]);
int client(int argc, char *argv[]);

#endif 
